@@ -53,11 +53,14 @@
 int verbose;
 
 // User-defined random seed
-size_t random_seed;
+size_t             random_seed;
+std::random_device default_seed_dev;
 // Overall probability of running conventional tests
 double test_prob;
 // Probability of running tests from the emulation suite
 double emulation_prob;
+// Probability of running unit tests
+double unittest_prob;
 // Modifier for probability of running tests with complex interleaved data
 double complex_interleaved_prob_factor;
 // Modifier for probability of running tests with real data
@@ -81,6 +84,9 @@ size_t ramgb;
 // Device memory limitation for tests (GiB):
 size_t vramgb;
 
+// Number of hip devices to use.
+int ngpus{};
+
 // Allow skipping tests if there is a runtime error
 bool skip_runtime_fails;
 // But count the number of failures
@@ -166,11 +172,8 @@ void init_gtest_flags()
     std::swap(temp_list_tests, testing::GTEST_FLAG(list_tests));
 }
 
-void precompile_test_kernels(const std::string& precompile_file)
+std::vector<std::string> tokens_to_run()
 {
-    std::cout << "precompiling test kernels...\n";
-    WorkQueue<std::string> tokenQueue;
-
     init_gtest_flags();
 
     std::vector<std::string> tokens;
@@ -218,6 +221,15 @@ void precompile_test_kernels(const std::string& precompile_file)
             }
         }
     }
+    return tokens;
+}
+
+void precompile_test_kernels(const std::string& precompile_file)
+{
+    std::cout << "precompiling test kernels...\n";
+    WorkQueue<std::string> tokenQueue;
+
+    auto tokens = tokens_to_run();
 
     std::random_device dev;
     std::mt19937       dist(dev());
@@ -306,24 +318,25 @@ int main(int argc, char* argv[])
         "      HP - hermitian planar\n"
         "\n"
         "Usage"};
-
-    // Override CLI11 help to print after later CLI11 options that are defined, and allow gtest's
-    // help.
-    // After removing the stage-1 options, individual options are set to null (even if set), but we
-    // can still capture the behaviour by using a flag.
-
-    for(auto opt : app.get_options())
-    {
-        app.remove_option(opt);
-    }
+    // Override CLI11 help to print it along gtest's help
+    app.set_help_flag("");
+    const auto opt_help = app.add_flag("-h, --help", "Produces this help message");
     app.add_option("-v, --verbose", verbose, "Print out detailed information for the tests")
         ->default_val(0);
     app.add_option("--nrand", n_random_tests, "Number of extra randomized tests")->default_val(0);
+
+    app.add_option("--ngpus", ngpus, "Number of GPUs to use per rank")
+        ->default_val(-1)
+        ->check(CLI::NonNegativeNumber);
+    app.add_option("--gpus", n_random_tests, "Number of extra randomized tests")->default_val(0);
     app.add_option("--test_prob", test_prob, "Probability of running individual tests")
         ->default_val(1.0)
         ->check(CLI::Range(0.0, 1.0));
+    app.add_option("--unittest_prob", unittest_prob, "Probability of running individual unit tests")
+        ->default_val(1.0)
+        ->check(CLI::Range(0.0, 1.0));
     app.add_option(
-           "--emulation_prob", test_prob, "Probability of running individual emulation tests")
+           "--emulation_prob", emulation_prob, "Probability of running individual emulation tests")
         ->default_val(1.0)
         ->check(CLI::Range(0.0, 1.0));
     app.add_option("--real_prob",
@@ -411,7 +424,7 @@ int main(int argc, char* argv[])
             if(mp_lib == fft_params::fft_mp_lib_none)
             {
                 std::cout << "--mp_launch requires an mp library (see mp_lib in --help).\n";
-                std::exit(-1);
+                std::exit(EXIT_FAILURE);
             }
         })
         ->needs("--mp_lib");
@@ -420,8 +433,9 @@ int main(int argc, char* argv[])
         ->each([&](const std::string&) {
             // The objective is to have an test that takes about 5 minutes, so just set the
             // probability per test to a small value to achieve this result.
-            test_prob      = 0.001;
-            emulation_prob = 0.01;
+            test_prob      = 0.0005;
+            emulation_prob = 0.005;
+            unittest_prob  = 0.2;
             n_random_tests = 10;
         });
 
@@ -429,53 +443,8 @@ int main(int argc, char* argv[])
         manual_params.run_callbacks = true;
     });
 
-    {
-        // We explicitly scope opt_seed so that the object falls out of scope before the final
-        // parsing of the command line arguments.  Otherwise, the second parsing would mark the
-        // option as not having been specified, which can get rather confusing.
-
-        auto opt_seed = app.add_option(
-            "--seed", random_seed, "Random seed; if unset, use an actual random seed");
-
-        // Try parsing initial args that will be used to configure tests.
-        // Allow extras to pass on gtest and rocFFT arguments without error.
-        app.allow_extras();
-        try
-        {
-            app.parse(argc, argv);
-        }
-        catch(const CLI::ParseError& e)
-        {
-            return app.exit(e);
-        }
-
-        if(!*opt_seed)
-        {
-            std::cout << "Generating random seed: ";
-            std::random_device dev;
-            random_seed = dev();
-            std::cout << random_seed << "\n";
-        }
-    }
-
-    app.set_help_flag("");
-    auto opt_help = app.add_flag("-h, --help", "Produces this help message");
-
-    std::vector<std::string> remaining_args = app.remaining();
-    // Google test ignores the first element, so add something there so that it parses all of hte
-    // arguments that we want it to parse.:
-    remaining_args.insert(remaining_args.begin(), argv0);
-    // NB: If we initialize gtest first, then it removes all of its own command-line
-    // arguments and sets argc and argv correctly;
-    std::vector<char*> carg;
-    for(std::string& s : remaining_args)
-    {
-        carg.push_back(&s[0]);
-    }
-    carg.push_back(NULL);
-    decltype(argc) cargc = carg.size() - 1;
-    ::testing::InitGoogleTest(&cargc, carg.data());
-
+    app.add_option("--seed", random_seed, "Random seed; if unset, use an actual random seed")
+        ->default_val(default_seed_dev());
     // Filename for fftw and fftwf wisdom.
     std::string fftw_wisdom_filename;
 
@@ -488,6 +457,9 @@ int main(int argc, char* argv[])
     // Full path to bitwise repro database file
     std::string repro_db_path;
 
+    // Bool option to just print tokens and exit
+    bool printtokens{false};
+
     // Declare the supported options. Some option pointers are declared to track passed opts.
     app.add_flag("--version", "Print queryable version information from the rocfft library")
         ->each([](const std::string&) {
@@ -514,6 +486,11 @@ int main(int argc, char* argv[])
                      "Type of transform:\n0) complex forward\n1) complex inverse\n2) real "
                      "forward\n3) real inverse")
         ->default_val(fft_transform_type_complex_forward);
+    non_token
+        ->add_option("--auto_allocation",
+                     manual_params.auto_allocate,
+                     "rocFFT's auto-allocation behavior: \"on\", \"off\", or \"default\"")
+        ->default_val("default");
     non_token
         ->add_option("--precision",
                      manual_params.precision,
@@ -576,6 +553,7 @@ int main(int argc, char* argv[])
                    precompile_file,
                    "Precompile kernels to a file for all test cases before running tests")
         ->default_val("");
+    app.add_flag("--printtokens", printtokens, "Print test tokens to scheduled to be run and exit");
     // Default value is set in fft_params.h based on if device-side PRNG was enabled.
     app.add_option("-g, --inputGen",
                    manual_params.igen,
@@ -584,36 +562,57 @@ int main(int argc, char* argv[])
                    "2) linearly-spaced sequence (device)\n"
                    "3) linearly-spaced sequence (host)");
 
-    // Parse rest of args and catch any errors here
+    // Try parsing initial args that will be used to configure tests
+    // Allow extras to pass on gtest arguments without error
+    app.allow_extras();
     try
     {
-        app.parse(cargc, carg.data());
+        app.parse(argc, argv);
     }
     catch(const CLI::ParseError& e)
     {
         return app.exit(e);
     }
 
+    // extract remaining arguments for subsequent gtest initialization
+    std::vector<std::string> remaining_args = app.remaining();
+    std::string              gtest_help_opt = "--help";
+    // NB: If we initialize gtest first, then it removes all of its own command-line
+    // arguments and sets argc and argv correctly;
+    std::vector<char*> gtest_argv;
+    gtest_argv.insert(gtest_argv.begin(), argv[0]);
+    for(std::string& s : remaining_args)
+    {
+        gtest_argv.push_back(&s[0]);
+    }
+    if(*opt_help)
+    {
+        // make sure gtest prints its help as well
+        gtest_argv.push_back(&gtest_help_opt[0]);
+    }
+    gtest_argv.push_back(NULL);
+    decltype(argc) gtest_argc = gtest_argv.size() - 1;
+    ::testing::InitGoogleTest(&gtest_argc, gtest_argv.data()); // gtest-relevant args are removed
+
     if(*opt_help)
     {
         std::cout << app.help() << "\n";
         return EXIT_SUCCESS;
     }
-
-    // Ensure there are no leftover options used by neither gtest nor CLI11
-    const auto leftover_args = app.remaining();
-    if(!leftover_args.empty())
+    // no help was used, gtest_argc is expected to be 1 at this point. If not, some of the
+    // used options were not recognized at all
+    if(gtest_argc > 1)
     {
         std::cout << "Unrecognised option(s) found:\n  ";
-        for(auto i : leftover_args)
-            std::cout << i << " ";
+        for(auto i = 1; i < gtest_argc; i++)
+            std::cout << gtest_argv[i] << " ";
         std::cout << "\nRun with --help for more information.\n";
         return EXIT_FAILURE;
     }
 
     std::cout << "half epsilon: " << half_epsilon << "\tsingle epsilon: " << single_epsilon
-              << "\tdouble epsilon: " << double_epsilon << "\n";
-    std::cout << "Random seed: " << random_seed << "\n";
+              << "\tdouble epsilon: " << double_epsilon << std::endl;
+    std::cout << "Random seed: " << random_seed << std::endl;
 
     // If precompiling, tell rocFFT to use the specified cache file
     // to write kernels to
@@ -628,9 +627,11 @@ int main(int argc, char* argv[])
     }
 
     rocfft_setup();
-    char v[256];
-    rocfft_get_version_string(v, 256);
-    std::cout << "rocFFT version: " << v << "\n";
+    {
+        char v[256];
+        rocfft_get_version_string(v, 256);
+        std::cout << "rocFFT version: " << v << std::endl;
+    }
 
 #ifdef FFTW_MULTITHREAD
     fftw_init_threads();
@@ -641,6 +642,7 @@ int main(int argc, char* argv[])
 
     // Set host memory limit from command-line options
     host_memory::singleton().set_limit_gbytes(ramgb);
+    std::cout << "Host memory limit: " << ramgb << " GiB" << std::endl;
 
     if(use_fftw_wisdom)
     {
@@ -693,7 +695,7 @@ int main(int argc, char* argv[])
 
     if(!test_token.empty())
     {
-        std::cout << "Reading fft params from token:\n" << test_token << "\n";
+        std::cout << "Reading fft params from token:\n" << test_token << std::endl;
 
         try
         {
@@ -701,8 +703,8 @@ int main(int argc, char* argv[])
         }
         catch(...)
         {
-            std::cout << "Unable to parse token.\n";
-            return 1;
+            std::cout << "Unable to parse token." << std::endl;
+            return EXIT_FAILURE;
         }
     }
     else
@@ -729,6 +731,17 @@ int main(int argc, char* argv[])
     if(!precompile_file.empty())
         precompile_test_kernels(precompile_file);
 
+    if(printtokens)
+    {
+        std::cout << "Tokens:" << std::endl;
+        const auto tokens = tokens_to_run();
+        for(const auto& token : tokens)
+        {
+            std::cout << token << std::endl;
+        }
+        return EXIT_SUCCESS;
+    }
+
     auto retval = RUN_ALL_TESTS();
 
     if(use_fftw_wisdom)
@@ -749,7 +762,8 @@ int main(int argc, char* argv[])
     const auto test_minutes
         = std::chrono::duration_cast<std::chrono::minutes>(test_duration - test_hours);
     std::cout << "Test suite took " << test_hours.count() << " hours " << test_minutes.count()
-              << " minutes\n\n";
+              << " minutes\n"
+              << std::endl;
 
     std::cout << "half precision max l-inf epsilon: " << max_linf_eps_half << "\n";
     std::cout << "half precision max l2 epsilon:     " << max_l2_eps_half << "\n";
@@ -758,8 +772,12 @@ int main(int argc, char* argv[])
     std::cout << "double precision max l-inf epsilon: " << max_linf_eps_double << "\n";
     std::cout << "double precision max l2 epsilon:     " << max_l2_eps_double << "\n";
     std::cout << "Number of runtime issues: " << n_hip_failures << "\n";
+    std::cout << "Number of successful tests: "
+              << ::testing::UnitTest::GetInstance()->successful_test_count() << "\n";
+    std::cout << "Number of skipped tests: "
+              << ::testing::UnitTest::GetInstance()->skipped_test_count() << "\n";
 
-    std::cout << "\nRandom seed: " << random_seed << "\n";
+    std::cout << "\nRandom seed: " << random_seed << std::endl;
 
     return retval;
 }
