@@ -22,7 +22,7 @@
 #define TWIDDLES_H
 
 #include "../../../shared/gpubuf.h"
-#include "rocfft.h"
+#include "rocfft/rocfft.h"
 #include <vector>
 
 static const size_t       LTWD_BASE_DEFAULT       = 8;
@@ -32,13 +32,25 @@ static const unsigned int TWIDDLES_MAX_RADICES    = 8;
 gpubuf twiddles_create(size_t                     N,
                        size_t                     length_limit,
                        rocfft_precision           precision,
-                       const char*                gpu_arch,
+                       const hipDeviceProp_t&     deviceProp,
                        size_t                     largeTwdBase,
                        bool                       attach_halfN,
                        const std::vector<size_t>& radices,
                        unsigned int               deviceId);
-gpubuf twiddles_create_2D(
-    size_t N1, size_t N2, rocfft_precision precision, const char* gpu_arch, unsigned int deviceId);
+gpubuf twiddles_create_2D(size_t                     N1,
+                          size_t                     N2,
+                          rocfft_precision           precision,
+                          const hipDeviceProp_t&     deviceProp,
+                          bool                       attach_halfN,
+                          bool                       attach_halfN2,
+                          const std::vector<size_t>& radices1,
+                          const std::vector<size_t>& radices2,
+                          unsigned int               deviceId);
+
+gpubuf twiddles_pp_create(size_t                 N,
+                          rocfft_precision       precision,
+                          const hipDeviceProp_t& deviceProp,
+                          unsigned int           deviceId);
 
 void twiddle_streams_cleanup();
 
